package com.mh.user.config.wechat;

import lombok.extern.slf4j.Slf4j;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;
import java.util.Map;
import java.util.TreeMap;

/**
 * @Author : Rainbow
 * @date : 2024-09-14
 */
@Slf4j
public class WechatSignUtil {

    /**
     * SHA-1加密
     * @param params
     * @return
     */
    public static String generateSignature(Map<String, String> params) {
        // 使用TreeMap进行自然排序（按照key的ASCII码排序）
        TreeMap<String, String> sortedParams = new TreeMap<>(params);

        // 拼接字符串
        StringBuilder stringBuilder = new StringBuilder();
        for (Map.Entry<String, String> entry : sortedParams.entrySet()) {
            if (stringBuilder.length() > 0) {
                stringBuilder.append("&");
            }
            stringBuilder.append(entry.getKey()).append("=").append(entry.getValue());
        }
        String string1 = stringBuilder.toString();

        // SHA-1加密
        try {
            // 获取SHA-1实例
            MessageDigest digest = MessageDigest.getInstance("SHA-1");

            // 将输入字符串转换为字节数组
            byte[] encodedHash = digest.digest(string1.getBytes());

            // 可选：将字节数组转换为Base64字符串，便于显示和传输
            String base64Encoded = Base64.getEncoder().encodeToString(encodedHash);
            log.info("SHA-1 (Base64 Encoded): {}", base64Encoded);

            // 或者，直接以16进制形式输出
            StringBuilder hexString = new StringBuilder();
            for (byte b : encodedHash) {
                String hex = Integer.toHexString(0xff & b);
                if(hex.length() == 1) hexString.append('0');
                hexString.append(hex);
            }
            log.info("SHA-1 (Hexadecimal): {}", hexString);
            return hexString.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("SHA-1 algorithm not found", e);
        }
    }
}
