package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.user.annotation.FieldParam;
import com.mh.user.annotation.SysLogger;
import com.mh.user.annotation.TranslationDict;
import com.mh.user.entity.DeviceParamsEntity;
import com.mh.user.service.DeviceParamsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 设备参数管理控制层
 * @date 2024-08-23 08:53:03
 */
@RestController
@RequestMapping("/deviceParams")
public class DevicesParamsController {

    @Autowired
    private DeviceParamsService deviceParamsService;

    @TranslationDict({
            @FieldParam(type = "mt_type", targetField = "mtType", targetFieldValueClazz = Integer.class)
    })
    @SysLogger(value="设备参数管理",optDesc = "查询设备参数管理")
    @PostMapping("/findByPage")
    public PageResult queryGatewayManage(@RequestBody PageRequest pageRequest) {
        return deviceParamsService.queryByPage(pageRequest);
    }

    @SysLogger(value="设备参数管理",optDesc = "编辑设备参数管理")
    @PostMapping("/update")
    public HttpResult updateProInfo(@RequestBody DeviceParamsEntity entity) {
        deviceParamsService.update(entity);
        return HttpResult.ok();
    }

    @SysLogger(value="设备参数管理",optDesc = "根据项目id查询对应信息")
    @GetMapping("/findById")
    public HttpResult findById(@RequestParam("id") Long id) {
        DeviceParamsEntity entity = deviceParamsService.findById(id);
        return HttpResult.ok(entity);
    }

    @SysLogger(value="设备参数管理",optDesc = "添加设备参数管理")
    @PostMapping("/save")
    public HttpResult saveGw(@RequestBody DeviceParamsEntity entity) {
        deviceParamsService.save(entity);
        return HttpResult.ok();
    }

    @SysLogger(value="设备参数管理",optDesc = "删除设备参数管理")
    @GetMapping("/deleteById")
    public HttpResult deleteProInfo(@RequestParam String id) {
        deviceParamsService.delete(id);
        return HttpResult.ok();
    }
    
}
