package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.user.annotation.FieldParam;
import com.mh.user.annotation.SysLogger;
import com.mh.user.annotation.TranslationDict;
import com.mh.user.entity.GatewayManageEntity;
import com.mh.user.service.GatewayManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 网关管理接口控制类
 * @date 2024-08-22 17:19:21
 */
@RestController
@RequestMapping("/gw")
public class GatewayManageController {

    @Autowired
    private GatewayManageService gatewayManageService;

    @TranslationDict({
            @FieldParam(type = "operator_type", targetField = "operatorType", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "communication_type", targetField = "communicationType", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "project_name", targetField = "projectId", targetFieldValueClazz = Integer.class),
    })
    @SysLogger(value="网关管理信息",optDesc = "查询网关管理信息")
    @PostMapping("/findByPage")
    public PageResult queryGatewayManage(@RequestBody PageRequest pageRequest) {
        return gatewayManageService.queryByPage(pageRequest);
    }

    @SysLogger(value="网关管理信息",optDesc = "编辑网关管理信息")
    @PostMapping("/update")
    public HttpResult updateProInfo(@RequestBody GatewayManageEntity gatewayManageEntity) {
        gatewayManageService.update(gatewayManageEntity);
        return HttpResult.ok();
    }

    @SysLogger(value="网关管理信息",optDesc = "根据项目id查询对应信息")
    @GetMapping("/findById")
    public HttpResult findById(@RequestParam("id") Long id) {
        GatewayManageEntity gatewayManageEntity = gatewayManageService.findById(id);
        return HttpResult.ok(gatewayManageEntity);
    }

    @SysLogger(value="网关管理信息",optDesc = "添加网关管理信息")
    @PostMapping("/save")
    public HttpResult saveGw(@RequestBody GatewayManageEntity gatewayManageEntity) {
        gatewayManageService.save(gatewayManageEntity);
        return HttpResult.ok();
    }

    @SysLogger(value="网关管理信息",optDesc = "删除网关管理信息")
    @GetMapping("/deleteById")
    public HttpResult deleteProInfo(@RequestParam String id) {
        gatewayManageService.delete(id);
        return HttpResult.ok();
    }

}
