package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.user.annotation.CacheChanges;
import com.mh.user.annotation.FieldParam;
import com.mh.user.annotation.SysLogger;
import com.mh.user.annotation.TranslationDict;
import com.mh.user.entity.MeterManageEntity;
import com.mh.user.service.MeterManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 仪表管理控制层
 * @date 2024-08-23 08:53:03
 */
@RestController
@RequestMapping("/mtManage")
public class MeterManageController {

    @Autowired
    private MeterManageService meterManageService;

    @TranslationDict({@FieldParam(type = "mt_type", targetField = "mtType", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "communication_type", targetField = "communicationType", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "data_type", targetField = "dataType", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "protocol_type", targetField = "protocolType", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "project_name", targetField = "projectId", targetFieldValueClazz = Integer.class),
            @FieldParam(type = "grade", targetField = "grade", targetFieldValueClazz = Integer.class),
    })
    @SysLogger(value="仪表管理信息",optDesc = "查询仪表管理信息")
    @PostMapping("/findByPage")
    public PageResult queryGatewayManage(@RequestBody PageRequest pageRequest) {
        return meterManageService.queryByPage(pageRequest);
    }

    @CacheChanges(value = "device_code_params")
    @SysLogger(value="仪表管理信息",optDesc = "编辑仪表管理信息")
    @PostMapping("/update")
    public HttpResult updateProInfo(MeterManageEntity entity) {
        meterManageService.update(entity);
        return HttpResult.ok();
    }

    @SysLogger(value="仪表管理信息",optDesc = "根据项目id查询对应信息")
    @GetMapping("/findById")
    public HttpResult findById(@RequestParam("id") Long id) {
        MeterManageEntity entity = meterManageService.findById(id);
        return HttpResult.ok(entity);
    }

    @CacheChanges(value = "device_code_params")
    @SysLogger(value="仪表管理信息",optDesc = "添加仪表管理信息")
    @PostMapping("/save")
    public HttpResult saveGw(MeterManageEntity entity) {
        meterManageService.save(entity);
        return HttpResult.ok();
    }

//    @CacheChanges(value = "device_code_params")
    @SysLogger(value="仪表管理信息",optDesc = "删除仪表管理信息")
    @GetMapping("/deleteById")
    public HttpResult deleteProInfo(@RequestParam String id) {
        meterManageService.delete(id);
        return HttpResult.ok();
    }

    @SysLogger(value="仪表管理信息",optDesc = "判断是否包含热量计")
    @GetMapping("/isHotCollection")
    public HttpResult queryIsHotCollection(@RequestParam String systemId,@RequestParam String projectId) {
        return HttpResult.ok(meterManageService.queryIsHotCollection(systemId,projectId));
    }
    
}
