package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.user.annotation.CacheChanges;
import com.mh.user.annotation.SysLogger;
import com.mh.user.model.SysDict;
import com.mh.user.service.SysDictService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 系统字典管理
 * @date 2024-08-23 10:59:09
 */
@RestController
@RequestMapping("/dict")
public class SysDictController {

    @Autowired
    private SysDictService sysDictService;

    @SysLogger(value="系统字典管理",optDesc = "查询系统字典管理")
    @PostMapping("/findByPage")
    public PageResult queryGatewayManage(@RequestBody PageRequest pageRequest) {
        return sysDictService.findPage(pageRequest);
    }

    @CacheChanges(value = "sys_dict")
    @SysLogger(value="系统字典管理",optDesc = "编辑系统字典管理")
    @PostMapping("/update")
    public HttpResult update(SysDict entity) {
        sysDictService.updateByPrimaryKey(entity);
        return HttpResult.ok();
    }

    @SysLogger(value="系统字典管理",optDesc = "根据项目id查询对应信息")
    @GetMapping("/findById")
    public HttpResult findById(@RequestParam("id") Long id) {
        SysDict entity = sysDictService.findById(id);
        return HttpResult.ok(entity);
    }

    @CacheChanges(value = "sys_dict")
    @SysLogger(value="系统字典管理",optDesc = "添加系统字典管理")
    @PostMapping("/save")
    public HttpResult save(SysDict entity) {
        sysDictService.save(entity);
        return HttpResult.ok();
    }

    @CacheChanges(value = "sys_dict")
    @SysLogger(value="系统字典管理",optDesc = "删除系统字典管理")
    @GetMapping("/deleteById")
    public HttpResult delete(@RequestParam Long id) {
        sysDictService.deleteByPrimaryKey(id);
        return HttpResult.ok();
    }

    @SysLogger(value="系统字典管理",optDesc = "根据设备类型查询系统字典管理")
    @GetMapping("/findByType")
    public HttpResult findByType(@RequestParam String type) {
        return HttpResult.ok(sysDictService.findByType(type));
    }

    @SysLogger(value="系统字典管理",optDesc = "根据设备类型查询系统字典管理")
    @GetMapping("/queryTypes")
    public HttpResult queryTypes() {
        return HttpResult.ok(sysDictService.queryTypes());
    }

}
