package com.mh.user.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.OrderMessageEntity;
import org.apache.ibatis.annotations.*;

import java.util.List;

@Mapper
public interface DeviceCodeParamMapper extends BaseMapper<DeviceCodeParamEntity> {

    //插入指令参数
    @Insert("insert into device_code_param(device_addr,device_name,device_type,data_com,baud_rate,brand,fun_code,register_addr,create_time)" +
            "values(#{deviceAddr},#{deviceName},#{deviceType},#{dataCom},#{baudrate},#{brand},#{funCode},#{registerAddr},getDate())")
    void insertDeviceCodeParam(DeviceCodeParamEntity deviceCodeParamEntity);

    //查询指令参数,传入IP和端口
    @Results(id="rs", value = {
            @Result(property="id",column="id"),
            @Result(property="deviceAddr",column="device_addr"),
            @Result(property="deviceType",column="device_type"),
            @Result(property="strData",column="str_data"),
            @Result(property="dataPort",column="data_port"),
            @Result(property="baudRate",column="baud_rate"),
            @Result(property ="brand",column ="brand"),
            @Result(property ="funCode",column ="fun_code"),
            @Result(property ="registerAddr",column ="register_addr"),
            @Result(property ="registerName",column ="register_name"),
            @Result(property ="deviceName",column ="device_name"),
            @Result(property ="createTime",column ="create_time"),
            @Result(property ="grade",column ="grade"),
            @Result(property ="digit",column ="digit"),
            @Result(property ="projectId",column ="project_id"),
            @Result(property ="mmId",column ="mm_id"),
            @Result(property ="dataType",column ="data_type"),
            @Result(property ="protocolType",column ="protocol_type")
    })
    @Select("select id, " +
            "device_addr, " +
            "device_type, " +
            "str_data, " +
            "data_port, " +
            "baud_rate, " +
            "brand, " +
            "fun_code, " +
            "register_addr, " +
            "device_name, " +
            "create_time, " +
            "grade, " +
            "project_id, " +
            "digit, " +
            "register_name," +
            "mm_id," +
            "data_type," +
            "protocol_type " +
            "   from  device_code_param  where data_port=#{gatewayPort} order by device_type ")
    List<DeviceCodeParamEntity> queryCodeParam(@Param("gatewayPort") String gatewayPort);

    //查询指令参数,传入串口
    @ResultMap("rs")
    @Select("select t1.* from  device_code_param t1 join gateway_manage t2  on t1.data_com=t2.data_com  where t2.data_com=#{comName}")
    List<DeviceCodeParamEntity> queryCodeParam2(@Param("comName") String comName);

    //查询所有指令参数
    @ResultMap("rs")
    @Select("select * from  device_code_param order by data_com,device_type")
    List<DeviceCodeParamEntity> queryCodeParam3();

    //查询所有指令参数
    @ResultMap("rs")
    @Select("select * from  device_code_param  where  project_id=#{projectId} and  device_type=#{deviceType} and  device_addr=#{deviceAddr} and register_addr=#{registerAddr}")
    DeviceCodeParamEntity queryDeviceCodeParam(@Param("projectId") String projectId,
                                               @Param("deviceType") String deviceType,
                                               @Param("deviceAddr") String deviceAddr,
                                               @Param("registerAddr") String registerAddr);

    //查询记录数
    @Select("select  count(*) from device_code_param  where  project_id=#{projectId} ")
    int queryCount(@Param("projectId") String projectId);

    //查询记录数,所有项目
    @Select("select  count(*) from device_code_param ")
    int queryCount2();

    // 批量插入生成指令参数入数据库
    @Insert("<script>" +
            "insert into device_code_param(" +
            "device_addr, " +
            "device_name, " +
            "device_type, " +
            "data_port, " +
            "baud_rate, " +
            "brand, " +
            "fun_code, " +
            "register_addr," +
            "register_name," +
            "create_time," +
            "project_id," +
            "parity," +
            "digit," +
            "grade," +
            "str_data," +
            "mm_id," +
            "data_type," +
            "protocol_type" +
            ")" +
            "values " +
            "<foreach collection='deviceCodeParamEntityList' item='item' separator=','>" +
            "(#{item.deviceAddr},#{item.deviceName},#{item.deviceType},#{item.dataPort},#{item.baudRate}," +
            "#{item.brand},#{item.funCode},#{item.registerAddr},#{item.registerName},getDate(),#{item.projectId}," +
            "#{item.parity},#{item.digit},#{item.grade},#{item.strData},#{item.mmId},#{item.dataType},#{item.protocolType})" +
            "</foreach>" +
            "</script>")
    void insertDeviceCodeParamList(@Param("deviceCodeParamEntityList") List<DeviceCodeParamEntity> deviceCodeParamEntityList);

    //查询插入
    @Insert("insert into device_code_param(device_addr,device_name,device_type,data_port,baud_rate,parity,brand,create_time,project_id,digit,grade) select device_addr,device_name,device_type,data_port,baud_rate,parity,brand,getDate(),project_id,2,grade  from  device_code " +
            " where  (device_type='电表' or device_type='水表') and  project_id=#{projectId} ")
    void selectInsertDeviceCodeParam(@Param("projectId") String projectId);

    @Delete("delete from device_code_param where project_id=#{projectId} ")
    void deleteDeviceCodeParam(@Param("projectId") long projectId);
}
