package com.mh.user.mapper;

import com.mh.user.dto.ScreenRequestParamDTO;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 大屏服务mapper
 * @date 2024-09-24 14:54:06
 */
public interface ScreenMapper {

    @Select("with projectmttypes as ( " +
            "    select  " +
            "        project_id, " +
            "        count(distinct mt_type) as distinct_mt_types " +
            "    from  " +
            "        meter_manage mm " +
            "    where  " +
            "        mm.mt_type in (1, 2) " +
            "        and mt_is_sum = 1 " +
            "        and mm.grade between 40 and 49 " +
            "        and mm.system_id = 1 " +
            "    group by  " +
            "        project_id " +
            "    having  " +
            "        count(distinct mt_type) = 2 " +
            ") " +
            "select  " +
            "    pi2.id, " +
            "    pi2.project_name, " +
            "    t.mt_type, " +
            "    t.cur_value, " +
            "    t.cur_time " +
            "from  " +
            "    project_info pi2 " +
            "left join projectmttypes pmt on pi2.id = pmt.project_id " +
            "left join ( " +
            "    select  " +
            "        project_id, " +
            "        mt_type, " +
            "        sum(isnull(cur_value, 0)) as cur_value, " +
            "        cur_time " +
            "    from  " +
            "        meter_manage mm " +
            "    where  " +
            "        mm.mt_type in (1, 2) " +
            "        and mt_is_sum = 1 " +
            "        and mm.grade between 140 and 149 " +
            "        and mm.system_id = 1 " +
            "        and datepart(hour, mm.cur_time) = datepart(hour, getdate()) " +
            "    group by  " +
            "        project_id, " +
            "        mt_type, " +
            "        cur_time " +
            ") t on pi2.id = t.project_id " +
            "where  " +
            "    pmt.project_id is not null; ")
    List<Map<String, Object>> queryNowData();

    @Select("select " +
            " pi2.project_name, " +
            " pi2.longitude, " +
            " pi2.latitude, " +
            " isnull(t.fault, 0) as fault " +
            "from " +
            " project_info pi2 " +
            "left join ( " +
            "select " +
            " project_id, " +
            " system_id, " +
            " case " +
            "  when sum(cur_value) > 0 then 1 " +
            "  else 0 " +
            " end as fault " +
            "from " +
            " meter_manage mm " +
            "where " +
            " mm.system_id = 1 " +
            " and mm.grade between 50 and 59 " +
            " and system_id = 1 " +
            "group by " +
            " project_id, " +
            " system_id " +
            ") t on pi2.id = t.project_id and pi2.system_id = t.system_id  ")
    List<Map<String, Object>> queryProjectData();

    @Select("select  " +
            "  cur_date as curDate,  " +
            "  elect,  " +
            "  cl,  " +
            "  cop  " +
            "from  " +
            "  energy_data_day edm  " +
            "where  " +
            "  cur_date between #{startTime} and #{endTime}  " +
            "  and project_id = #{projectId}  " +
            "order by  " +
            "  cur_date")
    List<Map<String, Object>> queryMonthData(ScreenRequestParamDTO screenRequestParamDTO);

    /**
     * 
     * @param screenRequestParamDTO
     * @return
     */
    @Select("<script>" +
            "WITH ProjectMTType1 AS ( " +
            "SELECT " +
            " project_id " +
            "FROM " +
            " meter_manage mm " +
            "WHERE " +
            " mm.mt_type = 1 " +
            " AND mt_is_sum = 1 " +
            " AND mm.grade BETWEEN 40 AND 49 " +
            " AND mm.system_id = 1 " +
            "GROUP BY " +
            " project_id " +
            "HAVING " +
            " COUNT(DISTINCT mt_type) = 1 " +
            "), " +
            "ProjectMTType2 AS ( " +
            "SELECT " +
            " project_id " +
            "FROM " +
            " meter_manage mm " +
            "WHERE " +
            " mm.mt_type = 2 " +
            " AND mt_is_sum = 1 " +
            " AND mm.grade BETWEEN 40 AND 49 " +
            " AND mm.system_id = 1 " +
            "GROUP BY " +
            " project_id " +
            "HAVING " +
            " COUNT(DISTINCT mt_type) = 1 " +
            ") " +
            "SELECT " +
            " pi2.id, " +
            " pi2.project_name as projectName, " +
            " isnull(edd.elect, " +
            " 0) as elect " +
            "FROM " +
            " ProjectMTType1 pmt1 " +
            "left join project_info pi2 on " +
            " pi2.id = pmt1.project_id " +
            "left join energy_data_day edd on " +
            " edd.project_id = pmt1.project_id " +
            " and edd.cur_date between #{startTime} and #{endTime} " +
            "WHERE " +
            " NOT EXISTS ( " +
            " SELECT " +
            "  1 " +
            " FROM " +
            "  ProjectMTType2 pmt2 " +
            " WHERE " +
            "  pmt2.project_id = pmt1.project_id " +
            "    ) " +
            "<if test='projectId != null and projectId != \"\"'>" +
            "   and pi2.id = #{projectId} " +
            "</if>" +
            "</script>")
    List<Map<String, Object>> savingEnergy(ScreenRequestParamDTO screenRequestParamDTO);

    @Select("select " +
            " t1.cur_date as curDate, " +
            " t1.sumValue as curValue, " +
            " isnull(t2.sumValue, 0) as lastValue, " +
            " case " +
            "  when t2.sumValue > 0 then concat(convert(decimal(18, 2),(t1.sumValue - t2.sumValue)/ t2.sumValue * 100), '%') " +
            "  when (t1.sumValue - t2.sumValue) = 0 then '0.00%' " +
            "  else '同期无数据' " +
            " end yoy " +
            "from " +
            " ( " +
            " select " +
            "  cur_date, " +
            "  sum(elect) as sumValue " +
            " from " +
            "  energy_data_day edd " +
            " where " +
            "  edd.cur_date between #{startTime} and #{endTime} " +
            "  and edd.project_id = #{projectId} " +
            " group by " +
            "  cur_date " +
            ") t1 " +
            "left join ( " +
            " select " +
            "  convert(varchar(10), " +
            "  dateadd(YEAR, 1, cur_date), " +
            "  120) as cur_date , " +
            "  case  " +
            "   when elect is null then '同期无数据' " +
            "   else sum(elect) " +
            "  end " +
            "  sumValue  " +
            " from " +
            "  energy_data_day edd " +
            " where " +
            "  edd.cur_date between #{lastStartTime} and #{lastEndTime} " +
            "  and edd.project_id = #{projectId} " +
            " group by " +
            "  cur_date, elect " +
            ") t2 on " +
            " t1.cur_date = t2.cur_date order by t1.cur_date ")
    List<Map<String, Object>> savingYoyEnergyData(@Param("projectId") String projectId,
                                                  @Param("startTime") String startTime,
                                                  @Param("endTime") String endTime,
                                                  @Param("lastStartTime") String lastStartTime,
                                                  @Param("lastEndTime") String lastEndTime);
    @Select("select " +
            " pi2.project_name, " +
            " t1.other_name " +
            "from " +
            " project_info pi2 " +
            "join ( " +
            " select " +
            "  project_id, " +
            "  other_name " +
            " from " +
            "  meter_manage mm " +
            " where " +
            "  grade between 50 and 59 " +
            "  and cur_value = 1 " +
            "  and day(cur_time) = day(getdate()) " +
            " group by " +
            "  project_id, " +
            "  other_name " +
            " ) t1 on " +
            " pi2.id = t1.project_id ")
    List<Map<String, Object>> queryFaultList();
}
