package com.mh.user.service.chillers.impl;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.user.constants.Constant;
import com.mh.user.entity.*;
import com.mh.user.mapper.DevicesManageMapper;
import com.mh.user.mapper.MeterManageMapper;
import com.mh.user.mapper.ProjectInfoMapper;
import com.mh.user.mapper.chillers.ChillersMapper;
import com.mh.user.service.chillers.ChillersService;
import com.mh.user.utils.DateUtil;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.TimeDifferenceUtil;
import com.mh.user.vo.HostListVO;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author ljf
 * @title ：
 * @description ： 实现类
 * @updateTime 2020-05-20
 * @throws ：
 */
@Service
public class ChillersServiceImpl implements ChillersService {

    // 构造方法注入
    private final ChillersMapper chillersMapper;
    private List<HostValue> hostValueList;
    double chillerWaterInt = 0.00;     // 冷冻水回水温度 0A
    double chillerWaterOut = 0.00;     // 冷冻水出水温度 0B
    double coldWaterInt = 0.00;        // 冷却水回水温度 0E
    double coldWaterOut = 0.00;        // 冷却水出水温度 0F

    private final DevicesManageMapper devicesManageMapper;

    private final MeterManageMapper meterManageMapper;

    private final ProjectInfoMapper projectInfoMapper;

    public ChillersServiceImpl(ChillersMapper chillersMapper,
                               List<HostValue> hostValueList,
                               DevicesManageMapper devicesManageMapper,
                               MeterManageMapper meterManageMapper, ProjectInfoMapper projectInfoMapper) {
        this.chillersMapper = chillersMapper;
        this.hostValueList = hostValueList;
        this.devicesManageMapper = devicesManageMapper;
        this.meterManageMapper = meterManageMapper;
        this.projectInfoMapper = projectInfoMapper;
    }

    @Override
    public List<HostValue> queryTempValue(String deviceType) {
        hostValueList.clear();
        if (deviceType.equalsIgnoreCase("other")) {
            List<ChillersEntity> chillersEntityList = getRunningChillers();
            if (chillersEntityList.size() == 2) {
                // 获取对应的冷冻水、冷却水温度
                List<ChillersEntity> chillersEntityList1 = chillersMapper.getChillersTemp(chillersEntityList.get(0).getDeviceCode());
                List<ChillersEntity> chillersEntityList2 = chillersMapper.getChillersTemp(chillersEntityList.get(1).getDeviceCode());
                for (int i = 0; i < chillersEntityList1.size(); i++) {
                    String registerAddr = chillersEntityList1.get(i).getRegisterAddress();
                    if (registerAddr.equalsIgnoreCase("0A")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        chillerWaterInt = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0B")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        chillerWaterOut = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0E")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        coldWaterInt = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0F")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        coldWaterOut = Double.parseDouble(String.valueOf(bigDecimal));
                    }
                }
                for (int i = 0; i < chillersEntityList2.size(); i++) {
                    String registerAddr = chillersEntityList2.get(i).getRegisterAddress();
                    if (registerAddr.equalsIgnoreCase("0A")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        chillerWaterInt = Double.parseDouble(String.valueOf(bigDecimal)) + chillerWaterInt;
                    } else if (registerAddr.equalsIgnoreCase("0B")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        chillerWaterOut = Double.parseDouble(String.valueOf(bigDecimal)) + chillerWaterOut;
                    } else if (registerAddr.equalsIgnoreCase("0E")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        coldWaterInt = Double.parseDouble(String.valueOf(bigDecimal)) + coldWaterInt;
                    } else if (registerAddr.equalsIgnoreCase("0F")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        coldWaterOut = Double.parseDouble(String.valueOf(bigDecimal)) + coldWaterOut;
                    }
                }
                // 四舍五入，保留两位小数
                BigDecimal b2 = BigDecimal.valueOf(chillerWaterOut / 2);
                chillerWaterOut = b2.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                BigDecimal b1 = BigDecimal.valueOf(chillerWaterInt / 2);
                chillerWaterInt = b1.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                BigDecimal b4 = BigDecimal.valueOf(coldWaterOut / 2);
                coldWaterOut = b4.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                BigDecimal b3 = BigDecimal.valueOf(coldWaterInt / 2);
                coldWaterInt = b3.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();

            } else if (chillersEntityList.size() == 1) {
                // 获取对应的冷冻水、冷却水温度
                List<ChillersEntity> chillersEntityList1 = chillersMapper.getChillersTemp(chillersEntityList.get(0).getDeviceCode());
                for (int i = 0; i < chillersEntityList1.size(); i++) {
                    String registerAddr = chillersEntityList1.get(i).getRegisterAddress();
                    if (registerAddr.equalsIgnoreCase("0A")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        chillerWaterInt = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0B")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        chillerWaterOut = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0E")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        coldWaterInt = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0F")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        coldWaterOut = Double.parseDouble(String.valueOf(bigDecimal));
                    }
                }
            } else if (chillersEntityList.size() == 0) {    // update by ljf on 2021-01-11
                // 获取对应的冷水机组
                List<ChillersEntity> chillersEntityList3 = getStopChillers();
                // 获取对应的冷冻水、冷却水温度
                List<ChillersEntity> chillersEntityList1 = chillersMapper.getChillersTemp(chillersEntityList3.get(0).getDeviceCode());
                List<ChillersEntity> chillersEntityList2 = chillersMapper.getChillersTemp(chillersEntityList3.get(1).getDeviceCode());
                for (int i = 0; i < chillersEntityList1.size(); i++) {
                    String registerAddr = chillersEntityList1.get(i).getRegisterAddress();
                    if (registerAddr.equalsIgnoreCase("0A")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        chillerWaterInt = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0B")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        chillerWaterOut = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0E")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        coldWaterInt = Double.parseDouble(String.valueOf(bigDecimal));
                    } else if (registerAddr.equalsIgnoreCase("0F")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList1.get(i).getLastValue());
                        coldWaterOut = Double.parseDouble(String.valueOf(bigDecimal));
                    }
                }
                for (int i = 0; i < chillersEntityList2.size(); i++) {
                    String registerAddr = chillersEntityList2.get(i).getRegisterAddress();
                    if (registerAddr.equalsIgnoreCase("0A")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        chillerWaterInt = Double.parseDouble(String.valueOf(bigDecimal)) + chillerWaterInt;
                    } else if (registerAddr.equalsIgnoreCase("0B")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        chillerWaterOut = Double.parseDouble(String.valueOf(bigDecimal)) + chillerWaterOut;
                    } else if (registerAddr.equalsIgnoreCase("0E")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        coldWaterInt = Double.parseDouble(String.valueOf(bigDecimal)) + coldWaterInt;
                    } else if (registerAddr.equalsIgnoreCase("0F")) {
                        BigDecimal bigDecimal = new BigDecimal(chillersEntityList2.get(i).getLastValue());
                        coldWaterOut = Double.parseDouble(String.valueOf(bigDecimal)) + coldWaterOut;
                    }
                }
                // 四舍五入，保留两位小数
                BigDecimal b2 = BigDecimal.valueOf(chillerWaterOut / 2);
                chillerWaterOut = b2.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                BigDecimal b1 = BigDecimal.valueOf(chillerWaterInt / 2);
                chillerWaterInt = b1.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                BigDecimal b4 = BigDecimal.valueOf(coldWaterOut / 2);
                coldWaterOut = b4.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                BigDecimal b3 = BigDecimal.valueOf(coldWaterInt / 2);
                coldWaterInt = b3.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            }
            HostValue hostValue = new HostValue();
            hostValue.setId(1);
            hostValue.setType("chilledOutWater");
            hostValue.setValue(chillerWaterOut);
            hostValueList.add(hostValue);
            HostValue hostValue1 = new HostValue();
            hostValue1.setId(2);
            hostValue1.setType("chilledInWater");
            hostValue1.setValue(chillerWaterInt);
            hostValueList.add(hostValue1);
            HostValue hostValue2 = new HostValue();
            hostValue2.setId(3);
            hostValue2.setType("coolingOutWater");
            hostValue2.setValue(coldWaterOut);
            hostValueList.add(hostValue2);
            HostValue hostValue3 = new HostValue();
            hostValue3.setId(4);
            hostValue3.setType("coolingInWater");
            hostValue3.setValue(coldWaterInt);
            hostValueList.add(hostValue3);
        } else if (deviceType.equalsIgnoreCase("hostOne")) {

        } else if (deviceType.equalsIgnoreCase("hostTwo")) {

        } else if (deviceType.equalsIgnoreCase("town")) {

        }
        return hostValueList;
    }

    // 查询冷水机组进出水温度两者的平均值

    @Override
    public List<ChillersEntity> getRunningChillers() {
        return chillersMapper.getRunningChillers();
    }

    @Override
    public List<ChillersEntity> getStopChillers() {
        return chillersMapper.getStopChillers();
    }

    @Override
    public List<ChillersEntity> queryChillersByOther(int grade, String ddcAddr) {
        return chillersMapper.queryChillersByOther(grade, ddcAddr);
    }

    @Override
    public int insertChillersHistory(ChillersEntity chillersEntity) {
        try {
            System.out.println(chillersEntity.toString());
            // 判断是否大于等于最近插入时间的30分钟，
            String lastTimeStr = queryLastChillersHistory(chillersEntity);
            if (lastTimeStr == null || lastTimeStr.equalsIgnoreCase("")) {
                // 格式化时间分钟为15,30,45,00
                chillersEntity.setLastTime(DateUtil.formatDateStr(chillersEntity.getLastTime()));
                // 大于就插入数据库，否则就更新基表安装信息的表
                return chillersMapper.insertChillersHistory(chillersEntity);
            } else {
                int dValue = TimeDifferenceUtil.dValue(lastTimeStr);
                if (dValue >= 30) {
                    // 格式化时间分钟为15,30,45,00
                    chillersEntity.setLastTime(DateUtil.formatDateStr(chillersEntity.getLastTime()));
                    // 大于就插入数据库，否则就更新基表安装信息的表
                    return chillersMapper.insertChillersHistory(chillersEntity);
                }
            }
            // 不管时间差如何都进行更新基表安装信息
            chillersMapper.updateChillerRegisterAddr(chillersEntity);
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
        return 0;
    }

    @Override
    public void updateChillerRegisterAddr(ChillersEntity chillersEntity) {
        chillersMapper.updateChillerRegisterAddr(chillersEntity);
    }

    @Override
    public void updateDataNow(ChillersEntity chillersEntity) {
        try {
            // 判断是否存在实时记录表
            int a = chillersMapper.queryDataNowByOthers(chillersEntity);
            if (a == 0) {
                // 不存在则进行插入处理
                chillersMapper.insertChillerDataNow(chillersEntity);
            } else {
                // 存在则进行更新处理
                chillersMapper.updateDataNow(chillersEntity);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public int batchUpdateRegisterAddr(List<ChillersEntity> chillersEntityLists) {
        return chillersMapper.updateRegisterAddr(chillersEntityLists);
    }

    @Override
    public List<TableDataEntity> queryTopData(String deviceName, int grade) {
        return chillersMapper.queryTopData(deviceName, grade);
    }

    @Override
    public List<TableInfoEntity> queryLeftAndRightData(String deviceName, int grade) {
        return chillersMapper.queryLeftAndRightData(deviceName, grade);
    }

    @Override
    public List<ChillersEntity> queryChillersDDC(int grade) {
        return chillersMapper.queryChillersDDC(grade);
    }

    @Override
    public void batchInsertChillersHistory(List<ChillersEntity> chillersEntityLists) {
        chillersMapper.batchInsertChillersHistory(chillersEntityLists);
    }

    @Override
    public String queryLastChillersHistory(ChillersEntity chillersEntity) {
        return chillersMapper.queryLastChillersHistory(chillersEntity);
    }

    @Override
    public List<String> queryDDCAddr(String ddcAddr) {
        return chillersMapper.queryDDCAddr(ddcAddr);
    }

    @Override
    public JSONArray queryChillersByProId(String systemId, String proId) {
        JSONArray resultJson = new JSONArray();
        QueryWrapper<DevicesManageEntity> deviceQueryWrapper = new QueryWrapper<>();
        if (systemId != null && !systemId.equalsIgnoreCase("")) {
            deviceQueryWrapper.eq("system_id", systemId);
        }
        if (proId != null && !proId.equalsIgnoreCase("")) {
            deviceQueryWrapper.eq("project_id", proId);
        }
        deviceQueryWrapper.eq("device_type", 0);
        List<DevicesManageEntity> devicesManageEntityList = devicesManageMapper.selectList(deviceQueryWrapper.orderByAsc("id"));
        if (devicesManageEntityList != null && !devicesManageEntityList.isEmpty()) {
            // 根据projectId进行分组
            Map<Long, List<DevicesManageEntity>> proDevicesMap =
                    devicesManageEntityList.stream().collect(Collectors.groupingBy(DevicesManageEntity::getProjectId));
            // 遍历分组后的数据
            for (Map.Entry<Long, List<DevicesManageEntity>> entry : proDevicesMap.entrySet()) {
                if (entry.getValue() != null && !entry.getValue().isEmpty()) {
                    // 根据key(projectId)获取对应的项目名称
                    String projectName = projectInfoMapper.selectById(entry.getKey()).getProjectName();
                    JSONObject proJson = new JSONObject();
                    proJson.put("name", projectName);
                    proJson.put("value", entry.getKey());

                    JSONArray children = new JSONArray();
                    List<DevicesManageEntity> devicesManageEntities = entry.getValue();
                    for (DevicesManageEntity devicesManageEntity : devicesManageEntities) {
                        JSONObject deviceJson = new JSONObject();
                        deviceJson.put("name", devicesManageEntity.getDeviceName());
                        deviceJson.put("value", devicesManageEntity.getId());
                        JSONArray mtArray = new JSONArray();
                        List<MeterManageEntity> meterManageEntities = meterManageMapper.selectList(
                                new QueryWrapper<MeterManageEntity>()
                                        .eq("device_id", devicesManageEntity.getId())
                                        .eq("project_id", devicesManageEntity.getProjectId())
                                        .orderByAsc("sort"));
                        if (meterManageEntities == null || meterManageEntities.isEmpty()) {
                            continue;
                        }
                        HostListVO hostListVO = new HostListVO();
                        matchingChillersData(meterManageEntities, hostListVO);
                        // 额定制冷量
                        hostListVO.setRatedRef(String.valueOf(devicesManageEntity.getRatedRefrigerationCapacity()));
                        // 额定功率
                        hostListVO.setRatedPower(String.valueOf(devicesManageEntity.getRatedInputPower()));
                        // 额定COP
                        hostListVO.setRatedCop(String.valueOf(devicesManageEntity.getRatedCop()));
                        mtArray.add(hostListVO);
                        deviceJson.put("children", mtArray);
                        children.add(deviceJson);
                    }
                    proJson.put("children", children);
                    resultJson.add(proJson);
                }
            }
        }
        return resultJson;
    }

    private void matchingChillersData(List<MeterManageEntity> meterManageEntities, HostListVO hostListVO) {
//        hostListVO.reset();
        for (MeterManageEntity meterManageEntity : meterManageEntities) {
            // 匹配HostListVO展示给前端
            String funCode = ExchangeStringUtil.addZeroForNum(meterManageEntity.getFuncCode(), 2);
            Long meterType = meterManageEntity.getMtType();

            String curValue = new BigDecimal(String.valueOf(meterManageEntity.getCurValue() == null ? 0.00 : meterManageEntity.getCurValue())).setScale(1, RoundingMode.HALF_UP).toString();
            switch (meterManageEntity.getRegisterAddr()) {
                case "0001":
                    if (funCode.equals("01")) {
                        hostListVO.setRunningStatus(String.valueOf(new BigDecimal(curValue).intValue()));
                    }
                    break;
                case "0003":
                    if (funCode.equals("03")) {
                        hostListVO.setEnteringTempSet(curValue);
                    }
                    break;
                case "0005":
                    if (funCode.equals("03")) {
                        hostListVO.setEvapLeavingChWTemp(curValue);
                    }
                    break;
                case "0006":
                    if (funCode.equals("03")) {
                        hostListVO.setEvapEnteringChWTemp(curValue);
                    }
                    break;
                case "0021":
                    if (funCode.equals("03") && meterType == 0) {
                        hostListVO.setEvapApproach(curValue);
                    }
                    break;
                case "0009":
                    if (funCode.equals("03")) {
                        hostListVO.setCondLeavingChWTemp(curValue);
                    }
                    break;
                case "0010":
                    if (funCode.equals("03")) {
                        hostListVO.setCondEnteringChWTemp(curValue);
                    }
                    break;
                case "0023":
                    if (funCode.equals("03") && meterType == 0) {
                        hostListVO.setCondApproach(curValue);
                    }
                    break;
                case "0139":
                    if (funCode.equals("03")) {
                        hostListVO.setRealTimeChillerAmps(curValue);
                    }
                    break;
                case "0013":
                    if (funCode.equals("03")) {
                        hostListVO.setRealTimePower(curValue);
                    }
                    break;
                case "0014":
                    if (funCode.equals("03") && meterType == 0) {
                        hostListVO.setRealTimeCop(curValue);
                    }
                    break;
                case "0122":
                    if (funCode.equals("03")) {
                        BigDecimal warn = new BigDecimal(curValue);
                        String warnStr = warn.compareTo(BigDecimal.ZERO) > 0 ? "主机报警" : "无报警";
                        String faultStr = warn.compareTo(BigDecimal.ZERO) > 0 ? "主机故障" : "无故障";
                        hostListVO.setRealTimeWarning(warnStr);
                        hostListVO.setRealTimeFault(faultStr);
                    }
                    break;
                default:
                    break;
            }
        }
    }

    @Override
    public JSONObject queryTotalData(String systemId, String proId, String deviceId) {
        JSONObject resultJson = new JSONObject();
        List<Map<String, Object>> resultMap = chillersMapper.queryTotalData(systemId, proId, deviceId);
        if (resultMap != null && !resultMap.isEmpty()) {
            for (Map<String, Object> map : resultMap) {
                Long mtType = (Long) map.get("mt_type");
                if (Objects.equals(mtType, Constant.COOLING_METER)) {
                    // 冷量计
                    resultJson.put("coolingUsed", new BigDecimal(String.valueOf(map.get("cur_value"))).setScale(1, RoundingMode.HALF_UP));
                } else if (Objects.equals(mtType, Constant.DIG_DISPLAY_METER) || Objects.equals(mtType, Constant.THREE_METER)) {
                    // 电量
                    resultJson.put("eleUsed", new BigDecimal(String.valueOf(map.get("cur_value"))).setScale(1, RoundingMode.HALF_UP));
                }
            }
            if (resultJson.get("eleUsed") == null) {
                resultJson.put("eleUsed", 0.0);
            }
            if (resultJson.get("coolingUsed") == null) {
                resultJson.put("coolingUsed", 0.0);
            }
            resultJson.put("carbonEmission",
                    resultJson.getString("eleUsed").isEmpty() ? 0.0
                            : new BigDecimal(Constant.CARBON_EMISSION_FACTOR * Double.parseDouble(resultJson.getString("eleUsed")))
                            .setScale(1, RoundingMode.HALF_UP)
                            .doubleValue());
            BigDecimal coolingUsed = resultJson.getBigDecimal("coolingUsed") == null ? BigDecimal.ZERO : resultJson.getBigDecimal("coolingUsed");
            BigDecimal ele = resultJson.getBigDecimal("eleUsed") == null ? BigDecimal.ZERO : resultJson.getBigDecimal("eleUsed");
            if (coolingUsed.compareTo(BigDecimal.ZERO) > 0
                    && ele.compareTo(BigDecimal.ZERO) > 0) {
                resultJson.put("cop", coolingUsed
                        .divide(new BigDecimal(String.valueOf(resultJson.getBigDecimal("eleUsed"))), 1, RoundingMode.HALF_UP));
            } else if (ele.compareTo(BigDecimal.ZERO) == 0) {
                resultJson.put("cop", 0.0);
            }
            return resultJson;
        }
        return resultJson;
    }

    @Override
    public JSONObject queryChartData(String systemId, String proId, String deviceId) {
        JSONObject resultJson = new JSONObject();
        List<Map<String, Object>> resultList = chillersMapper.queryChartData(systemId, proId, deviceId, Constant.CARBON_EMISSION_FACTOR);
        // 将 resultList 转换为包含时间戳和 COP 的字符串数组
        String[] curDate = resultList.stream()
                .map(map -> (String) map.get("curDate"))
                .toArray(String[]::new);
        BigDecimal[] elect = resultList.stream()
                .map(map -> new BigDecimal(String.valueOf(map.get("elect"))))
                .toArray(BigDecimal[]::new);
        BigDecimal[] carbonEmission = resultList.stream()
                .map(map -> new BigDecimal(String.valueOf(map.get("carbonEmission"))))
                .toArray(BigDecimal[]::new);
        BigDecimal[] cl = resultList.stream()
                .map(map -> new BigDecimal(String.valueOf(map.get("cl"))))
                .toArray(BigDecimal[]::new);
        BigDecimal[] cop = resultList.stream()
                .map(map -> new BigDecimal(String.valueOf(map.get("cop"))))
                .toArray(BigDecimal[]::new);
        resultJson.put("curDate", curDate);
        resultJson.put("elect", elect);
        resultJson.put("cl", cl);
        resultJson.put("cop", cop);
        resultJson.put("carbonEmission", carbonEmission);
        return resultJson;
    }
}
