package com.mh.user.service.impl;

import com.mh.common.http.HttpResult;
import com.mh.user.constants.Constant;
import com.mh.user.dto.DataResultDTO;
import com.mh.user.entity.*;
import com.mh.user.mapper.*;
import com.mh.user.service.DataResultService;
import com.mh.user.service.DealDataExceptionService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.MeterManageService;
import com.mh.user.utils.DateUtil;
import com.mh.user.utils.ExchangeStringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@Service
public class DataResultServiceImpl implements DataResultService {

    @Autowired
    DataResultMapper dataResultMapper;

    @Autowired
    DeviceInstallMapper deviceInstallMapper;

    @Autowired
    DeviceInstallService deviceInstallService;

    @Resource
    DataResultChMapper dataResultChMapper;

    @Resource
    DataResultClMapper dataResultClMapper;

    @Resource
    DataResultClBakMapper dataResultClBakMapper;


    @Resource
    DataResultChillerMapper dataResultChillerMapper;

    @Resource
    DataResultOneMiMapper dataResultOneMiMapper;

    @Resource
    DataResultFiveMiMapper dataResultFiveMiMapper;

    @Resource
    DataResultFifteenMiMapper dataResultFifteenMiMapper;

    @Resource
    DataResultNowMapper dataResultNowMapper;

    @Resource
    MeterManageMapper meterManageMapper;

    @Resource
    MeterManageService meterManageService;

    @Resource
    private DealDataExceptionService dealDataExceptionService;

    private final SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public void saveDataResult(DataResultEntity dataResultEntity) {

        try {
            //获取系统日期
            Date date = new Date();
            if (dataResultEntity.getCurDate() == null) {
                dataResultEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            MeterManageEntity meterManageEntity = meterManageMapper.selectDevice(dataResultEntity.getDeviceAddr(),
                    dataResultEntity.getDeviceType(),
                    dataResultEntity.getProjectId(),
                    dataResultEntity.getRegisterAddr());

            //判断是否有记录
            DataResultEntity data = new DataResultEntity();
            //通讯地址
            data.setDeviceAddr(dataResultEntity.getDeviceAddr()); //通讯地址
            data.setDeviceType(dataResultEntity.getDeviceType()); //设备类型
            String curDate = sdf1.format(dataResultEntity.getCurDate());
            data.setCurDate(sdf1.parse(curDate));//当前日期
            data.setCurValue(dataResultEntity.getCurValue());     //当前读数
            data.setProjectId(String.valueOf(meterManageEntity.getProjectId()));   //楼栋编号
            data.setRatio(BigDecimal.valueOf(meterManageEntity.getMtRatio()));             //倍率
            BigDecimal lastValue = meterManageEntity.getCurValue();       //安装基表上次读数
            BigDecimal calcValue = dataResultEntity.getCurValue().subtract(lastValue); //计算用量
            data.setCalcValue(calcValue);//用量
            if (meterManageEntity.getCurTime() != null) {//上次抄表日期
                String lastDate = sdf1.format(meterManageEntity.getCurTime());
                data.setLastDate(sdf1.parse(lastDate));
            } else {
                String lastDate = sdf1.format(date);
                data.setLastDate(sdf1.parse(lastDate));
            }
            data.setLastValue(lastValue);//上次读数
            data.setGrade(dataResultEntity.getGrade());//级别
            data.setRegisterAddr(dataResultEntity.getRegisterAddr());
            data.setRegisterName(dataResultEntity.getRegisterName());
            dataResultMapper.saveDataResult(data);         //插入新的记录
        } catch (Exception e) {
            log.error("保存数据异常", e);
        }
    }

    @Override
    public void updateDataResult(DataResultEntity dataResultEntity) {
        dataResultMapper.updateDataResult(dataResultEntity);
    }

    @Override
    public List<DataResultEntity> queryDataResult(String projectId, String startDate, String endDate, String deviceType, int page, int limit) {
        return dataResultMapper.queryDataResult(projectId, startDate, endDate, deviceType, page, limit);
    }

    @Override
    public int getDataResultCount(String projectId, String startDate, String endDate, String deviceType, int page, int limit) {
        return dataResultMapper.getDataResultCount(projectId, startDate, endDate, deviceType, page, limit);
    }

    //-------------------------------------------------------------------------------------
    //保存流量计用量
    @Override
    public void saveDataResultCl(DataResultClEntity dataResultClEntity) throws ParseException {
        try {
            //获取系统日期
            //从安装表获取设备信息
//            DeviceInstallEntity deviceInstallEntity = deviceInstallMapper.selectDevice(dataResultClEntity.getDeviceAddr(), dataResultClEntity.getDeviceType(), dataResultClEntity.getProjectId());
            MeterManageEntity meterManageEntity = meterManageMapper.selectDevice(dataResultClEntity.getDeviceAddr(),
                    dataResultClEntity.getDeviceType(),
                    dataResultClEntity.getProjectId(),
                    dataResultClEntity.getRegisterAddr());

            if (null == meterManageEntity) {
                return;
            }

            if (meterManageEntity.getCurTime() == null) {
                Date date = new Date();
                dataResultClEntity.setLastDate(date);
            }

            BigDecimal lastValue = meterManageEntity.getCurValue();
            BigDecimal curValue = dataResultClEntity.getCurValue();
            dataResultClEntity.setLastValue(lastValue);
            dataResultClEntity.setLastDate(meterManageEntity.getCurTime());
            dataResultClEntity.setRatio(BigDecimal.valueOf(meterManageEntity.getMtRatio())); //倍率
            dataResultClEntity.setCalcValue(curValue.subtract(lastValue).multiply(dataResultClEntity.getRatio()));

//            if ((curValue-lastValue)<1000 && (curValue-lastValue)>=0){
            if (curValue.compareTo(lastValue) >= 0) {
                dataResultClMapper.saveDataResultCl(dataResultClEntity);         //插入新的记录
                //修改安装表中lastValue,lastDate的值
//                deviceInstallMapper.updateLastValue(meterManageEntity.getId(), curValue, date);
            }

        } catch (Exception e) {
            log.error("保存流量计用量失败", e);
        }
    }

    //保存流量计用量
    @Override
    public void saveDataResultCl_bak(DataResultClEntity dataResultClEntity) throws ParseException {
        try {
            //获取系统日期
            if (dataResultClEntity.getCurDate() == null) {
                Date date = new Date();
                dataResultClEntity.setCurDate(date);
            }
            //从安装表获取设备信息
//            DeviceInstallEntity deviceInstallEntity = deviceInstallMapper.selectDevice(dataResultClEntity.getDeviceAddr(), dataResultClEntity.getDeviceType(), dataResultClEntity.getProjectId());
            MeterManageEntity meterManageEntity = meterManageMapper.selectDevice(dataResultClEntity.getDeviceAddr(),
                    dataResultClEntity.getDeviceType(),
                    dataResultClEntity.getProjectId(),
                    dataResultClEntity.getRegisterAddr());
            dataResultClEntity.setRatio(BigDecimal.valueOf(meterManageEntity.getMtRatio())); //倍率
            String curDate = DateUtil.dateToString(dataResultClEntity.getCurDate(), "yyyy-MM-dd HH:mm:ss");
            String deviceAddr = meterManageEntity.getMtCode();
            String registerAddr = dataResultClEntity.getRegisterAddr();
            String projectId = dataResultClEntity.getProjectId();
            int row = dataResultClBakMapper.selectDataResultChBakCount(curDate, deviceAddr, registerAddr, projectId);//无记录
            if (row == 0) {
                dataResultClBakMapper.saveDataResultCl_bak(dataResultClEntity);         //插入新的记录
            }

        } catch (Exception e) {
            log.error("保存流量计用量异常", e);
        }
    }

    @Override
    public void updateDataResultCl(DataResultClEntity dataResultClEntity) {
        dataResultClMapper.updateDataResultCl(dataResultClEntity);
    }

    //-------------------------------------------------------------------------------------
    @Override
    public void saveDataResultCh(DataResultChEntity dataResultChEntity) throws ParseException {
        try {
            //从安装表获取设备信息
            String deviceAddr = dataResultChEntity.getDeviceAddr();
            String registerAddr = dataResultChEntity.getRegisterAddr();
            String curDate = DateUtil.dateToString(dataResultChEntity.getCurDate(), "yyyy-MM-dd HH:mm:00");
            String projectId = dataResultChEntity.getProjectId();
            int row = dataResultChMapper.selectDataResultChCount(curDate, deviceAddr, registerAddr, projectId, dataResultChEntity.getGrade());//无记录
            if (row == 0) {
                dataResultChMapper.saveDataResultCh(dataResultChEntity);         //插入新的记录
            }

        } catch (Exception e) {
            log.error("保存冷暖器用量异常", e);
        }
    }

    @Override
    public void updateDataResultCh(DataResultChEntity dataResultChEntity) {
        dataResultChMapper.saveDataResultCh(dataResultChEntity);
    }

    @Override
    public List<DataResultChEntity> queryDataResultMi(String projectId, String startDate, String curDate, int page, int limit, int type, String registerAddr, String mtNum) {
        if (type == 0 && !projectId.equals("20")) {
            return dataResultOneMiMapper.queryDataResultOneMi(projectId, startDate, curDate, page, limit);
        } else if (type == 1 && !projectId.equals("20")) {
            return dataResultFiveMiMapper.queryDataResultFiveMi(projectId, startDate, curDate, page, limit, registerAddr, mtNum);
        } else if (type == 2 && !projectId.equals("20")) {
            return dataResultFifteenMiMapper.queryDataResultFifteenMi(projectId, startDate, curDate, page, limit, registerAddr, mtNum);
        } else if (projectId.equals("20") && (type == 1 || type == 2)) {
            // 需要两个珠江啤酒整合起来
            List<DataResultChEntity> data1 = new ArrayList<>();
            List<DataResultChEntity> data2 = new ArrayList<>();
            if (type == 1) {
                data1 = dataResultFiveMiMapper.queryDataResultFiveMi("14", startDate, curDate, page, limit, registerAddr, mtNum);
                data2 = dataResultFiveMiMapper.queryDataResultFiveMi("15", startDate, curDate, page, limit, registerAddr, mtNum);

            } else {
                data1 = dataResultFifteenMiMapper.queryDataResultFifteenMi("14", startDate, curDate, page, limit, registerAddr, mtNum);
                data2 = dataResultFifteenMiMapper.queryDataResultFifteenMi("15", startDate, curDate, page, limit, registerAddr, mtNum);


            }
            if ("0006".equals(registerAddr)) {
                // 合并两个列表并根据时间分组
                Map<String, DataResultChEntity> mergedMap = Stream.concat(data1.stream(), data2.stream())
                        .collect(Collectors.toMap(
                                entity -> DateUtil.dateToString(entity.getCurDate(), "yyyy-MM-dd HH:mm:ss"),
                                entity -> entity,
                                (entity1, entity2) -> {
                                    entity1.setCurValue(new BigDecimal(entity1.getCurValue()).add(new BigDecimal(entity2.getCurValue())
                                            .setScale(2, RoundingMode.HALF_UP)).toString());
                                    return entity1;
                                }
                        ));

                // 重新赋值项目名称并创建新的列表
                return mergedMap.values().stream()
                        .map(entity -> {
                            entity.setProjectName("河北珠江啤酒"); // 根据需要重新赋值项目名称
                            return entity;
                        })
                        .collect(Collectors.toList());
            } else {
                // 合并两个列表并根据时间分组
                Map<String, DataResultChEntity> mergedMap = Stream.concat(data1.stream(), data2.stream())
                        .collect(Collectors.toMap(
                                entity -> DateUtil.dateToString(entity.getCurDate(), "yyyy-MM-dd HH:mm:ss"),
                                entity -> entity,
                                (entity1, entity2) -> {
                                    entity1.setCurValue((new BigDecimal(entity1.getCurValue()).add(new BigDecimal(entity2.getCurValue())))
                                            .divide(new BigDecimal("2")).setScale(2, RoundingMode.FLOOR).toString());
                                    return entity1;
                                }
                        ));

                // 重新赋值项目名称并创建新的列表
                return mergedMap.values().stream()
                        .map(entity -> {
                            entity.setProjectName("河北珠江啤酒"); // 根据需要重新赋值项目名称
                            return entity;
                        })
                        .collect(Collectors.toList());
            }
        } else {
            return dataResultFifteenMiMapper.queryDataResultFifteenMi(projectId, startDate, curDate, page, limit, registerAddr, mtNum);
        }
    }

    @Override
    public int dataResultOneMiCount(String projectId, String startDate, String curDate) {
        return dataResultOneMiMapper.dataResultOneMiCount(projectId, startDate, curDate);
    }

    @Override
    public HttpResult queryDataResultMiData(String projectId, String curDate, int page, int limit, int type, String dateType) {
        try {
            if (projectId != null && !projectId.isEmpty()) {
                String startDate = "";
                curDate = curDate.substring(0, 16) + ":00";

                // 判断项目是否为冷量计项目
                int isHotCollection = meterManageService.queryIsHotCollection("", projectId);
                String momentCoolingStr = Constant.MOMENT_COOLING;
                String outTempStr = Constant.OUT_TEMP;
                String intTempStr = Constant.INT_TEMP;
                if (isHotCollection == 2 || projectId.equals("20")) {
                    momentCoolingStr = Constant.MOMENT_HOT;
                    outTempStr = Constant.TEMP;
                    intTempStr = Constant.PRE;
                }
                // 查询温度低的冷量计
                String registerAddr = meterManageMapper.selectRegisterAddr(projectId, intTempStr);
                if (dateType.equals("1小时")) {
                    startDate = ExchangeStringUtil.dateRoll(1, curDate);
                    List<DataResultChEntity> records = queryDataResultMi(projectId, startDate, curDate, page, limit, 0, momentCoolingStr, registerAddr);
                    int count = dataResultOneMiCount(projectId, startDate, curDate);
                    return HttpResult.ok(count, records);
                } else if (dateType.equals("2小时")) {
                    startDate = ExchangeStringUtil.dateRoll(2, curDate);
                    List<DataResultChEntity> records = queryDataResultMi(projectId, startDate, curDate, page, limit, 0, momentCoolingStr, registerAddr);
                    int count = dataResultOneMiCount(projectId, startDate, curDate);
                    return HttpResult.ok(count, records);
                } else if (dateType.equals("8小时") || dateType.equals("12小时") || dateType.equals("24小时")) {
                    int queryType = 1;
                    if (dateType.equals("8小时")) {
                        startDate = ExchangeStringUtil.dateRoll(8, curDate);
                    } else if (dateType.equals("12小时")) {
                        startDate = ExchangeStringUtil.dateRoll(12, curDate);
                    } else {
                        startDate = ExchangeStringUtil.dateRoll(24, curDate);
                        queryType = 2;
                    }

                    // 查询瞬时冷量
                    List<DataResultChEntity> records = queryDataResultMi(projectId, startDate, curDate, page, limit, queryType, momentCoolingStr, null);
                    // 查询进水温度值（及冷冻水出水温度）
                    List<DataResultChEntity> intTempRecords = queryDataResultMi(projectId, startDate, curDate, page, limit, queryType, outTempStr, registerAddr);
                    // 查询出水温度值（及冷冻水进水温度）
                    List<DataResultChEntity> outTempRecords = queryDataResultMi(projectId, startDate, curDate, page, limit, queryType, intTempStr, registerAddr);
                    // 拼接数据
                    // 将数据按时间分组
                    Map<String, DataResultChEntity> momentCoolingMap = records.stream()
                            .collect(Collectors.toMap(val -> {
                                return DateUtil.dateToString(val.getCurDate(), "yyyy-MM-dd HH:mm:ss");
                            }, entity -> entity));

                    Map<String, DataResultChEntity> intTempMap = intTempRecords.stream()
                            .collect(Collectors.toMap(val -> {
                                return DateUtil.dateToString(val.getCurDate(), "yyyy-MM-dd HH:mm:ss");
                            }, entity -> entity));

                    Map<String, DataResultChEntity> outTempMap = outTempRecords.stream()
                            .collect(Collectors.toMap(val -> {
                                return DateUtil.dateToString(val.getCurDate(), "yyyy-MM-dd HH:mm:ss");
                            }, entity -> entity));

                    // 合并数据
                    Set<String> allTimes = new HashSet<>(momentCoolingMap.keySet());
                    allTimes.addAll(intTempMap.keySet());
                    allTimes.addAll(outTempMap.keySet());

                    List<DataResultDTO> mergedRecords = new ArrayList<>();
                    for (String time : allTimes) {
                        DataResultChEntity momentCooling = momentCoolingMap.getOrDefault(time, new DataResultChEntity(time, "0"));
                        DataResultChEntity intTemp = intTempMap.getOrDefault(time, new DataResultChEntity(time, "0"));
                        DataResultChEntity outTemp = outTempMap.getOrDefault(time, new DataResultChEntity(time, "0"));

                        DataResultDTO mergedRecord = new DataResultDTO(
                                time,
                                momentCooling.getCurValue(),
                                intTemp.getCurValue(),
                                outTemp.getCurValue(),
                                momentCooling.getProjectName(),
                                momentCooling.getProjectId()
                        );

                        mergedRecords.add(mergedRecord);
                    }
                    // 定义时间格式
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    // 使用 Collections.sort 方法排序
                    Collections.sort(mergedRecords, (o1, o2) -> {
                        try {
                            Date date1 = sdf.parse(o1.getTime());
                            Date date2 = sdf.parse(o2.getTime());
                            return date2.compareTo(date1);
                        } catch (ParseException e) {
                            throw new IllegalArgumentException("Invalid date format", e);
                        }
                    });
                    int count = 0;
                    if (dateType.equals("24小时")) {
                        count = dataResultFifteenMiCount(projectId, startDate, curDate, momentCoolingStr, registerAddr);
                    } else {
                        count = dataResultFiveMiCount(projectId, startDate, curDate, momentCoolingStr, registerAddr);
                    }
                    dealDataExceptionService.dealFifteenInstantaneousExceptionData(mergedRecords);
                    return HttpResult.ok(count, mergedRecords);
                }
//                else if (dateType.equals("12小时")) {
//                    startDate = ExchangeStringUtil.dateRoll(12, curDate);
//                    List<DataResultChEntity> records = queryDataResultMi(projectId, startDate, curDate, page, limit, 1, Constant.MOMENT_COOLING);
//                    int count = dataResultFiveMiCount(projectId, startDate, curDate);
//                    return HttpResult.ok(count, records);
//                }
//                else if (dateType.equals("24小时")) {
//                    startDate = ExchangeStringUtil.dateRoll(24, curDate);
//                    List<DataResultChEntity> records = queryDataResultMi(projectId, startDate, curDate, page, limit, 2, Constant.MOMENT_COOLING);
//                    int count = dataResultFifteenMiCount(projectId, startDate, curDate);
//                    return HttpResult.ok(count, records);
//                }
                else { //1小时
                    startDate = ExchangeStringUtil.dateRoll(1, curDate);
                    List<DataResultChEntity> records = queryDataResultMi(projectId, startDate, curDate, page, limit, 0, momentCoolingStr, registerAddr);
                    int count = dataResultOneMiCount(projectId, startDate, curDate);
                    return HttpResult.ok(count, records);
                }
            } else {
                return HttpResult.error();
            }
        } catch (Exception e) {
            log.error("查询数据异常", e);
            return HttpResult.error();
        }
    }

    @Override
    public int dataResultFiveMiCount(String projectId, String startDate, String curDate, String registerAddr, String mtNum) {
        if ("20".equals(projectId)) {
            projectId = "14";
        }
        return dataResultFiveMiMapper.dataResultFiveMiCount(projectId, startDate, curDate, registerAddr, mtNum);
    }

    @Override
    public int dataResultFifteenMiCount(String projectId, String startDate, String curDate, String registerAddr, String mtNum) {
        if ("20".equals(projectId)) {
            projectId = "14";
        }
        return dataResultFifteenMiMapper.dataResultFifteenMiCount(projectId, startDate, curDate, registerAddr, mtNum);
    }

    @Override
    public List<DataResultChEntity> queryDataResultSec(String projectId, String startDate, String curDate) {
        return dataResultChMapper.queryDataResultCh(projectId, startDate, curDate);
    }

    @Override
    public void saveDataResultChiller(DataResultChEntity dataResultChEntity) {

        try {

            String deviceAddr = dataResultChEntity.getDeviceAddr();
            String registerAddr = dataResultChEntity.getRegisterAddr();

            String str = "";
            if (dataResultChEntity.getCurDate() != null) {
                str = sdf1.format(dataResultChEntity.getCurDate());
            } else {
                Date date = new Date();//获取系统日期
                str = sdf1.format(date);
            }

            String curDate = "";
            curDate = str.substring(0, 16) + ":00";
            String projectId = dataResultChEntity.getProjectId();
            dataResultChEntity.setCurDate(sdf1.parse(curDate));
            int row = dataResultChillerMapper.selectDataResultChillerCount(curDate, deviceAddr, registerAddr, projectId, dataResultChEntity.getFunCode());//无记录
            if (row == 0) {
                dataResultChillerMapper.saveDataResultChiller(dataResultChEntity);       //插入新的记录
            } else {
                dataResultChillerMapper.updateDataResultChiller(dataResultChEntity);     //修改
            }
        } catch (Exception e) {
            log.error("保存数据失败", e);
        }
    }

    @Override
    public void updateDataResultChiller(DataResultChEntity dataResultChEntity) {
        dataResultChillerMapper.updateDataResultChiller(dataResultChEntity);
    }

    @Override
    public void saveDataResultNow(DataResultChEntity dataResultChEntity) {
        String deviceAddr = dataResultChEntity.getDeviceAddr();
        String registerAddr = dataResultChEntity.getRegisterAddr();
        String deviceType = dataResultChEntity.getDeviceType();
        String projectId = dataResultChEntity.getProjectId();
        dataResultNowMapper.deleteDataResultNow(deviceAddr, deviceType, registerAddr, projectId);
        dataResultNowMapper.saveDataResultNow(dataResultChEntity);
    }

    @Override
    public void deleteDataResultNow(String deviceAddr, String deviceType, String registerAddr, String projectId) {
        dataResultNowMapper.deleteDataResultNow(deviceAddr, deviceType, registerAddr, projectId);
    }

    @Override
    public List<DataResultChEntity> selectDataResultNow(String deviceAddr, String deviceType, String projectId) {
        return dataResultNowMapper.selectDataResultNow(deviceAddr, deviceType, projectId);
    }

    @Override
    public List<DataResultChEntity> queryDataResultChiller(String projectId, String deviceAddr, String registerName, String startDate, String curDate, int page, int limit) {
        return dataResultChillerMapper.queryDataResultChiller(projectId, deviceAddr, registerName, startDate, curDate, page, limit);
    }

    @Override
    public int dataResultChillerCount(String projectId, String deviceAddr, String registerName, String startDate, String curDate) {
        return dataResultChillerMapper.dataResultChillerCount(projectId, deviceAddr, registerName, startDate, curDate);
    }
}
