package com.mh.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.common.utils.StringUtils;
import com.mh.user.entity.DevicesManageEntity;
import com.mh.user.mapper.DevicesManageMapper;
import com.mh.user.service.DevicesManageService;
import com.mh.user.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 资产设备管理实现类
 * @date 2024-08-23 10:25:06
 */
@Service
public class DevicesManageServiceImpl implements DevicesManageService {

    @Autowired
    private DevicesManageMapper devicesManageMapper;

    @Override
    public PageResult queryByPage(PageRequest pageRequest) {
        String systemID = StringUtils.getColumnFilterValue(pageRequest, "systemId");
        String projectId = StringUtils.getColumnFilterValue(pageRequest, "projectId");
        // 设置分页参数
        int pageNum = pageRequest.getPageNum();
        int pageSize = pageRequest.getPageSize();
        if (pageNum != 0) {
            PageHelper.startPage(pageNum, pageSize);
        }
        QueryWrapper<DevicesManageEntity> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(systemID)) {
            queryWrapper.eq("system_id", systemID);
        }
        if (!StringUtils.isBlank(projectId)) {
            queryWrapper.eq("project_id", projectId);
        }
        List<DevicesManageEntity> manageEntities = devicesManageMapper.selectList(queryWrapper.orderByDesc("create_time"));
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(manageEntities));
    }

    @Override
    public void update(DevicesManageEntity entity) {
        entity.setUpdateTime(new Date());
        devicesManageMapper.updateById(entity);
    }

    @Override
    public DevicesManageEntity findById(Long id) {
        return devicesManageMapper.selectById(id);
    }

    @Override
    public void save(DevicesManageEntity entity) {
        entity.setCreateBy(SecurityUtils.getUsername());
        entity.setCreateTime(new Date());
        devicesManageMapper.insert(entity);
    }

    @Override
    public void delete(String id) {
        devicesManageMapper.deleteById(id);
    }
}
