package com.mh.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.common.utils.StringUtils;
import com.mh.user.entity.GatewayManageEntity;
import com.mh.user.mapper.chillers.GatewayManageMapper;
import com.mh.user.service.GatewayManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 网关管理实现类
 * @date 2024-08-22 17:36:41
 */
@Service
public class GatewayManageServiceImpl implements GatewayManageService {

    @Autowired
    private GatewayManageMapper gatewayManageMapper;

    @Override
    public PageResult queryByPage(PageRequest pageRequest) {
        String systemID = StringUtils.getColumnFilterValue(pageRequest, "systemId");
        String projectId = StringUtils.getColumnFilterValue(pageRequest, "projectId");
        // 设置分页参数
        int pageNum = pageRequest.getPageNum();
        int pageSize = pageRequest.getPageSize();
        if (pageNum != 0) {
            PageHelper.startPage(pageNum, pageSize);
        }
        QueryWrapper<GatewayManageEntity> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(systemID)) {
            queryWrapper.eq("system_id", systemID);
        }
        if (!StringUtils.isBlank(projectId)) {
            queryWrapper.eq("project_id", projectId);
        }
        List<GatewayManageEntity> manageEntities = gatewayManageMapper.selectList(queryWrapper
                        .orderByDesc("create_time"));
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(manageEntities));
    }

    @Override
    public void update(GatewayManageEntity gatewayManageEntity) {
        gatewayManageEntity.setUpdateTime(new Date());
        gatewayManageMapper.updateById(gatewayManageEntity);
    }

    @Override
    public GatewayManageEntity findById(Long id) {
        return gatewayManageMapper.selectById(id);
    }

    @Override
    public void save(GatewayManageEntity gatewayManageEntity) {
        gatewayManageEntity.setCreateTime(new Date());
        gatewayManageMapper.insert(gatewayManageEntity);
    }

    @Override
    public void delete(String id) {
        gatewayManageMapper.deleteById(id);
    }

    @Override
    public String selectProjectName(String port) {
        return gatewayManageMapper.selectProjectName(port);
    }

    @Override
    public void updateGatewayManage2(String port) {
        gatewayManageMapper.updateGatewayManage2(port);
    }
}
