package com.mh.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.MeterManageEntity;
import com.mh.user.mapper.MeterManageMapper;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.service.MeterManageService;
import com.mh.user.utils.ExchangeStringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 仪表管理实现类
 * @date 2024-08-23 09:02:18
 */
@Service
public class MeterManageServiceImpl implements MeterManageService {

    @Autowired
    private MeterManageMapper meterManageMapper;

    @Autowired
    private DeviceCodeParamService deviceCodeParamService;

    @Override
    public List<MeterManageEntity> queryBySystemIdAndProjectId(String systemId, String projectId) {
        QueryWrapper<MeterManageEntity> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(systemId)) {
            queryWrapper.eq("system_id", systemId);
        }
        if (!StringUtils.isBlank(projectId)) {
            queryWrapper.eq("project_id", projectId);
        }
        // 0正常使用，1停用
        queryWrapper.eq("is_use", 0);
        return meterManageMapper.selectList(queryWrapper.orderByDesc("create_time"));
    }

    @Override
    public void updateDataById(Long mmId, String analysisData) {
        UpdateWrapper<MeterManageEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", mmId);
        // 判断analysisData是否为数值
        if (analysisData.matches("\\d+(\\.\\d+)?")) {
            updateWrapper.set("cur_value", analysisData);
        } else {
            updateWrapper.set("cur_value", "0.00");
        }
        updateWrapper.set("cur_time", new Date());
        meterManageMapper.update(updateWrapper);
    }

    @Override
    public PageResult queryByPage(PageRequest pageRequest) {
        String systemID = StringUtils.getColumnFilterValue(pageRequest, "systemId");
        String projectId = StringUtils.getColumnFilterValue(pageRequest, "projectId");
        // 设置分页参数
        int pageNum = pageRequest.getPageNum();
        int pageSize = pageRequest.getPageSize();
        if (pageNum != 0) {
            PageHelper.startPage(pageNum, pageSize);
        }
        QueryWrapper<MeterManageEntity> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(systemID)) {
            queryWrapper.eq("system_id", systemID);
        }
        if (!StringUtils.isBlank(projectId)) {
            queryWrapper.eq("project_id", projectId);
        }
        List<MeterManageEntity> meterManageEntities = meterManageMapper.selectList(queryWrapper.orderByDesc("create_time"));
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(meterManageEntities));
    }

    @Override
    public void update(MeterManageEntity entity) {
        entity.setUpdateTime(new Date());
        entity.setFuncCode(ExchangeStringUtil.addZeroForNum(entity.getFuncCode(), 2));
        // 判断仪表类型：
        // 电表格式化成12位
        if (entity.getMtType() == 1) {
            entity.setMtCode(ExchangeStringUtil.addZeroForNum(entity.getMtCode(), 12));
        }
        meterManageMapper.updateById(entity);
        // 更新对应的设备采集参数
        deviceCodeParamService.createCodeParams(entity);
    }

    @Override
    public MeterManageEntity findById(Long id) {
        return meterManageMapper.selectById(id);
    }

    @Override
    public void save(MeterManageEntity entity) {
        entity.setCreateTime(new Date());
        entity.setFuncCode(ExchangeStringUtil.addZeroForNum(entity.getFuncCode(), 2));
        // 电表格式化成12位
        if (entity.getMtType() == 1) {
            entity.setMtCode(ExchangeStringUtil.addZeroForNum(entity.getMtCode(), 12));
        }
        meterManageMapper.insert(entity);
        // 更新对应的设备采集参数
        deviceCodeParamService.createCodeParams(entity);
    }

    @Override
    public void delete(String id) {
        MeterManageEntity meterManageEntity = meterManageMapper.selectById(id);
        meterManageMapper.deleteById(id);
        // 更新对应的设备采集参数
        deviceCodeParamService.createCodeParams(meterManageEntity);
    }

    @Override
    public int queryIsHotCollection(String systemId, String projectId) {
        if (meterManageMapper.queryIsHotCollection(systemId, projectId, "0077", Constant.COOLING_METER) > 0) {
            // 判断是否存在热量计
            return 1;
        } else if (meterManageMapper.queryIsHotCollection(systemId, projectId, "0002", Constant.FLOW_METER) > 0 || projectId.equals("20")) {
            // 判断是否包含蒸汽流量计
            return 2;
        } else {
            return 0;
        }

    }
}
