package com.mh.user.service.wechat.impl;

import com.alibaba.fastjson2.JSONObject;
import com.github.benmanes.caffeine.cache.Cache;
import com.mh.user.config.wechat.WechatMpConfig;
import com.mh.user.entity.wechat.PushMsgEntity;
import com.mh.user.mapper.wechat.WechatMapper;
import com.mh.user.service.wechat.WechatService;
import com.mh.user.utils.CacheUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 微信服务类
 * @date 2024-11-28 17:05:16
 */
@Slf4j
@Service
public class WechatServiceImpl implements WechatService {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private WechatMpConfig wechatMpConfig;

    @Autowired
    private WechatMapper wechatMapper;

    @Override
    public List<Map<String, Object>> queryWechatUser(String projectId) {
        return wechatMapper.queryWechatUser(projectId);
    }

    public String getAccessToken(CacheUtil instance) {
        String tokenResult = restTemplate.getForObject(wechatMpConfig.getAccessTokenUrl(),String.class);
        JSONObject jsonObj = JSONObject.parseObject(tokenResult);
        if (jsonObj == null || !jsonObj.containsKey("access_token")) {
            return "";
        }
        String accessToken = jsonObj.get("access_token").toString();
        log.info("获取access_token==>{}", accessToken);
        instance.createCache("accessToken", accessToken , 7000, TimeUnit.SECONDS);
        return accessToken;
    }

    @Override
    public void PushMsg(PushMsgEntity pushMsgEntity) {
        // 判断cache中是否存在access_token
        CacheUtil instance = CacheUtil.getInstance();
        Cache<Object, Object> cache = instance.getCache("accessToken");
        String accessToken = "";
        if (cache != null ) {
            log.info("ConcurrentHashMap中存在access_token的cache，直接使用");
            // 再次判断是否存在access_token
            if (cache.getIfPresent("accessToken") == null) {
                log.info("cache中不存在access_token，重新获取");
                accessToken = getAccessToken(instance);
            } else {
                accessToken = Objects.requireNonNull(cache.getIfPresent("accessToken")).toString();
            }
        } else {
            String tokenResult = restTemplate.getForObject(wechatMpConfig.getAccessTokenUrl(),String.class);
            JSONObject jsonObj = JSONObject.parseObject(tokenResult);
            if (jsonObj == null || !jsonObj.containsKey("access_token")) {
                return;
            }
            accessToken = getAccessToken(instance);
        }
        if (accessToken.isEmpty()) {
            return;
        }
        // 封装推送内容
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("touser",pushMsgEntity.getTouser());
        jsonObject.put("template_id",pushMsgEntity.getTemplateId());
        jsonObject.put("url",pushMsgEntity.getUrl());
        JSONObject data = new JSONObject();
        data.put("first",pushMsgEntity.getFirst());
        data.put("thing1",pushMsgEntity.getKey1());
        // 判断模板id属于哪一个，切换对应的参数
        switch (pushMsgEntity.getTemplateId()) {
            case "qG3Odr74o4BRx8QigZuoiguLNSkw7QJ0zQI3LaHtMY8":
                // 推送单种设备消息
                data.put("character_string7", pushMsgEntity.getKey2());
                data.put("character_string8", pushMsgEntity.getKey3());
                data.put("character_string9", pushMsgEntity.getKey4());
                break;
            case "qG3Odr74o4BRx8QigZuoil3l3ZC2aAdDeO49robRLC8":
                // 推送用电量、用冷量、EER数据
                data.put("character_string2", pushMsgEntity.getKey2());
                data.put("character_string3", pushMsgEntity.getKey3());
                data.put("character_string4", pushMsgEntity.getKey4());
                break;
            default:
                break;

        }
        data.put("time5",pushMsgEntity.getKey5());
        data.put("remark",pushMsgEntity.getRemark());
        jsonObject.put("data",data);
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", "application/json");
        HttpEntity<Object> entity = new HttpEntity<Object>(jsonObject, headers);
        String result = restTemplate.postForObject(wechatMpConfig.getPushMsgUrl(accessToken),entity,String.class);
        log.info("推送结果：{}", result);
    }
}
