package com.mh.user.service.wechat.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.user.entity.wechat.WechatUserInfoEntity;
import com.mh.user.mapper.wechat.WechatUserInfoMapper;
import com.mh.user.service.wechat.WechatUserInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 微信服务实体实现类
 * @date 2024-11-27 17:03:56
 */
@Service
public class WechatUserInfoServiceImpl implements WechatUserInfoService {

    @Autowired
    private WechatUserInfoMapper wechatUserInfoMapper;

    @Transactional
    @Override
    public int existsUser(JSONObject jsonObject) {
        if (jsonObject == null || !jsonObject.containsKey("openid")) return 0;
        String openId = jsonObject.get("openid").toString();
        // 根据openId判断用户是否存在
        long count = wechatUserInfoMapper.selectCount(new QueryWrapper<WechatUserInfoEntity>().eq("open_id", openId));
        // count：0:未找到用户，1：有用户但未完成注册
        if (count != 0) {
            // 0：未注册；1：完成注册
            return 0;
        }

        WechatUserInfoEntity userEntity = new WechatUserInfoEntity();
        userEntity.setIsStatus(2);
        userEntity.setOpenId(openId);
        userEntity.setNickName(jsonObject.containsKey("nickname") ? jsonObject.get("nickname").toString() : null);
        if (jsonObject.containsKey("sex")) {
            userEntity.setSex(Integer.parseInt(jsonObject.get("sex").toString()));
        } else {
            userEntity.setSex(null);
        }
        userEntity.setProvince(jsonObject.containsKey("province") ? jsonObject.get("province").toString() : null);
        userEntity.setCountry(jsonObject.containsKey("country") ? jsonObject.get("country").toString() : null);
        userEntity.setCity(jsonObject.containsKey("city") ? jsonObject.get("city").toString() : null);
        userEntity.setHeadimgurl(jsonObject.containsKey("headimgurl") ? jsonObject.get("headimgurl").toString() : null);
        //用户不存在基本信息先入库
        wechatUserInfoMapper.insert(userEntity);
        return 0;
    }
}
