package com.mh.user.strategy;

import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.MeterManageEntity;
import com.mh.user.factory.Protocol;
import com.mh.user.utils.ExchangeStringUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 水表标准协议
 * @date 2024-09-12 14:04:01
 */
@Slf4j
public class CJ188ProtocolStrategy implements ProtocolStrategy {

    private static class SingletonHolder{
        private static final CJ188ProtocolStrategy INSTANCE = new CJ188ProtocolStrategy();
    }

    private CJ188ProtocolStrategy(){
        // 防止外部直接实例化
    }

    public static CJ188ProtocolStrategy getInstance(){
        return SingletonHolder.INSTANCE;
    }

    @Override
    public String createOrder(MeterManageEntity meterManageEntity) {
        log.info("水表标准协议：策略创建报文");
        if (StringUtils.isBlank(meterManageEntity.getRegisterAddr())) {
            return Constant.FAIL;
        }
        // 0 代表前面补充0,14 代表长度为14,d 代表参数为正数型
        String str = String.format("%014d", Long.parseLong(meterManageEntity.getRegisterAddr()));//基表通讯号
        // 转换位置
        str = ExchangeStringUtil.changePosition(str);
        // 拼接功能码
        str = "6810" + str +
                ExchangeStringUtil.addZeroForNum(meterManageEntity.getFuncCode(), 2)
                + "03"      // 控制码
                + meterManageEntity.getIdentifyCode()     // 水表读读数数据标识
                + "00";     // 序列号
        // 检验和
        String checkSum = ExchangeStringUtil.makeChecksum(str);
        str = "FEFEFE" + str + checkSum + "16";
        return str;
    }

    @Override
    public String analysisReceiveData(DeviceCodeParamEntity deviceCodeParamEntity, String receiveData) {
        log.info("水表标准协议：策略解析报文");
        String meterId = "";
        String data = "";
        if (receiveData.length() > 8) {
            String str1 = receiveData.substring(0, 8);
            String str2 = receiveData.substring(8);
            receiveData = str1.replace("FE", "") + str2;
        }
        String checkStr = receiveData.substring(0, receiveData.length() - 4);//减去校验码
        String checkNum = ExchangeStringUtil.makeChecksum(checkStr); //生成校验码
        if (checkNum.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4, receiveData.length() - 2))) {
            meterId = checkStr.substring(16, 18) + checkStr.substring(14, 16) + checkStr.substring(12, 14) + checkStr.substring(10, 12)// 表号
                    + checkStr.substring(8, 10) + checkStr.substring(6, 8) + checkStr.substring(4, 6);
            meterId = String.format("%014d", Long.parseLong(meterId));
            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < 4; i++) {
                String data1 = checkStr.substring(36 - 2 * (i + 1), 36 - 2 * i);
                stringBuilder.append(data1);
            }
            data = stringBuilder.toString();
            // 0 代表前面补充0，4 代表长度为4，d 代表参数为正数型
            data = String.format("%08d", Long.parseLong(data));
            data = data.substring(0, 6) + "." + data.substring(6, 8);
        } else {
            log.info("水表报文检验失败: {}", receiveData);
            return Constant.FAIL;
        }
        log.info("水表表号: {},水表读数：{}", meterId, data);
        return data;
    }

}
